import fs from 'fs'
import path from 'path';
import { loggingLocal, Tlog } from './router.js'
import { WebsocketRequestHandler } from 'express-ws'
import { getSingleLog } from './routerUtils.js';
import chalk, { ChalkInstance } from 'chalk'
import { colorName, colors } from './colors.js';
import { WebServerLogging } from './weblogging.js';


let sendNewLogToClient = function (props: Tlog) { }
export type Tlogging = 'fatal' | 'error' | 'warn' | 'important' | 'info' | 'success' | 'verbose'
type TlogLevel = 0 | 1 | 2 | 3 | 4 | 5 | 6
const loggingValues: { [key in Tlogging]: TlogLevel } = {
    fatal: 0,
    error: 1,
    warn: 2,
    important: 3,
    success: 4,
    info: 5,
    verbose: 6
}
export function getColoredText(text: string, color: colorName = 'black', ...notColoredTexts: string[]) {
    // const colors: { [key in Tcolor]: ChalkInstance } = {
    const coloredText = colors[color](text) + notColoredTexts
    return coloredText
}
/**
* @param {String} text text to return
* @param {*} color enums: [black, red, green, yellow, blue, magenta, cyan, white]
* @returns colored text
*/
function coloredConsoleLog(text: string, color: colorName = 'black', ...notColoredTexts: string[]) {
    const coloredText = getColoredText(text, color, ...notColoredTexts)
    console.log(coloredText)
}

class OurLogging extends WebServerLogging {
    loggingBrowserLength: number
    historyLogsLength: number
    #logLevel: number = 6
    #logFileName: string | undefined

    constructor(historyLogsLength?: number) {
        super()
        const defaultLength = 100

        this.loggingBrowserLength = 80
        this.historyLogsLength = historyLogsLength || defaultLength
    }
    setLogLevel(newLevel: Tlogging) { this.#logLevel = loggingValues[newLevel] }
    setBrowserMessagesAmount(amount: number) { this.loggingBrowserLength = amount }
    setLogfileName(name: string) { this.#logFileName = name }

    fatal(text: string, stackTrace: string = '') {
        if (!this.#aboveSelectedLogLevel('fatal')) return
        const currentTime = this.#getDateTime();

        let logText = `\n[ FATAL ERROR ] ${currentTime} | ${text}  ${stackTrace}`
        coloredConsoleLog('[ FATAL ERROR ] ', 'carmine', `${currentTime} | ${text} ${stackTrace}`)

        this.#addLogToGlobal(text + ' ' + stackTrace, 'fatal')
        this.#logTextToFile(logText)
    }

    error(text: string, stackTrace: string = '') {
        if (!this.#aboveSelectedLogLevel('error')) return
        const currentTime = this.#getDateTime()

        let logText = `\n[ ERROR ] ${currentTime} | ${text} ${stackTrace}`
        coloredConsoleLog('[ ERROR ] ', 'scarlet', `${currentTime} | ${text} ${stackTrace}`)

        this.#addLogToGlobal(text + ' ' + stackTrace, 'error')
        this.#logTextToFile(logText)
    }

    warn(...text: string[]) {
        if (!this.#aboveSelectedLogLevel('warn')) return
        const currentTime = this.#getDateTime()

        let logText = `\n[ WARNING ] ${currentTime} | ${text.join(' ')}`
        coloredConsoleLog('[ WARNING ] ', 'yellow', `${currentTime} | ${text.join(' ')}`)

        this.#addLogToGlobal(text.join(' '), 'warn')
        this.#logTextToFile(logText)
    }
    important(...text: string[]) {
        if (!this.#aboveSelectedLogLevel('important')) return
        const currentTime = this.#getDateTime()

        let logText = `\n[ IMPORTANT ] ${currentTime} | ${text.join(' ')}`
        coloredConsoleLog('[ IMPORTANT ] ', 'blue', `${currentTime} | ${text.join(' ')}`)

        this.#addLogToGlobal(text.join(' '), 'info')
        this.#logTextToFile(logText)
    }
    info(...text: string[]) {
        if (!this.#aboveSelectedLogLevel('info')) return
        const currentTime = this.#getDateTime()

        let logText = `\n[ INFO ] ${currentTime} | ${text.join(' ')}`
        coloredConsoleLog('[ INFO ] ', 'lightblue', `${currentTime} | ${text.join(' ')}`)

        this.#addLogToGlobal(text.join(' '), 'info')
        this.#logTextToFile(logText)
    }
    verbose(...text: string[]) {
        if (!this.#aboveSelectedLogLevel('verbose')) return
        const currentTime = this.#getDateTime()

        let logText = `\n[ VERBOSE ] ${currentTime} | ${text.join(' ')}`
        coloredConsoleLog('[ VERBOSE ] ', 'verbose_color', `${currentTime} | ${text.join(' ')}`)

        this.#addLogToGlobal(text.join(' '), 'verbose')
        this.#logTextToFile(logText)
    }

    success(...text: string[]) {
        if (!this.#aboveSelectedLogLevel('success')) return
        const currentTime = this.#getDateTime()

        let logText = `\n[ SUCCESS ] ${currentTime} | ${text.join(' ')}`
        coloredConsoleLog('[ SUCCESS ] ', 'green', `${currentTime} | ${text.join(' ')}`)

        this.#addLogToGlobal(text.join(' '), 'success')
        this.#logTextToFile(logText)
    }

    #addLogToGlobal(text: string, type: Tlogging) {
        if (loggingLocal.length > this.loggingBrowserLength)
            loggingLocal.shift()
        loggingLocal.push({ text, type, date: new Date().getTime() })
        sendNewLogToClient({ text, type, date: new Date().getTime() })
    }
    #getDateTime() {
        var currentDate = new Date()
        const date = currentDate.toLocaleDateString('en-US')
        const time = currentDate.toLocaleTimeString('en-US')
        return `${date} | ${time}`
    }
    #logTextToFile(logText: string) {
        if (!this.#logFileName)
            return

        if (!fs.existsSync(this.#logFileName)) {
            const dirname = path.dirname(this.#logFileName)
            fs.mkdirSync(dirname, { recursive: true })
        }

        try {
            fs.appendFileSync(this.#logFileName, logText)
        }
        catch (error) {
            console.error(coloredConsoleLog('LOGGING ERROR: ', 'scarlet', (<Error>error).message));
        }
    }
    #aboveSelectedLogLevel(testedLogLevel: Tlogging) {
        const selectedLogLevel = this.#logLevel
        const wantedLogLevel = loggingValues[testedLogLevel]
        return wantedLogLevel <= selectedLogLevel
    }
}
const logging = new OurLogging()
export default logging

export * from './router.js'

export const loggingWS: WebsocketRequestHandler = (ws) => {
    sendNewLogToClient = function (props: Tlog) {
        ws.send(JSON.stringify({
            newLog: getSingleLog(props.type, props.date, props.text)
        }))
    }
}
